package org.implab.gradle.containers;

import org.gradle.api.Plugin;
import org.gradle.api.Project;
import org.gradle.api.plugins.ExtraPropertiesExtension;
import org.implab.gradle.containers.tasks.BuildImage;
import org.implab.gradle.containers.tasks.PushImage;
import org.implab.gradle.containers.tasks.RunImage;
import org.implab.gradle.containers.tasks.SaveImage;
import org.implab.gradle.containers.tasks.TagImage;

public class ContainerBasePlugin implements Plugin<Project> {
    public static final String CONTAINER_EXTENSION_NAME = "container";

    private ContainerExtension containerExtension;

    ContainerExtension getContainerExtension() {
        if (containerExtension == null)
            throw new IllegalStateException();
        return containerExtension;
    }

    void exportClasses(Project project, Class<?>... classes) {
        ExtraPropertiesExtension extras = project.getExtensions().getExtraProperties();
        for (var clazz : classes)
            extras.set(clazz.getSimpleName(), clazz);
    }

    @Override
    public void apply(Project project) {

        containerExtension = project.getObjects().newInstance(ContainerExtension.class);

        containerExtension.getImageAuthority()
                .convention(project.provider(() -> (String) project.getProperties().get("imagesAuthority")));

        containerExtension.getImageGroup()
                .convention(project.provider(() -> (String) project.getProperties().get("imagesGroup")));

        containerExtension.getCliCmd()
                .convention(project
                        .provider(() -> (String) project.getProperties().get("containerCli"))
                        .orElse("docker"));

        project.getExtensions().add(CONTAINER_EXTENSION_NAME, containerExtension);

        exportClasses(
                project,
                BuildImage.class, PushImage.class, SaveImage.class, TagImage.class, RunImage.class);

    }

}
